@@ -38,8 +38,9 @@ class AVHandler:
             raise RuntimeError("Cannot add Receiver more than once")
         if receiver is not None:
             self._receiver = receiver
-            self._receiver.set_session(self._session)
-            self._receiver.get_video_codec()
+            # pylint: disable=protected-access
+            self._receiver._set_session(self._session)
+            self._receiver._get_video_codec()
 
     def set_cipher(self, cipher):
         """Set cipher. Schedules handler to run."""
@@ -61,12 +62,13 @@ class AVHandler:
                 self._receiver.handle_audio_data,
                 self._send_corrupt,
             )
-            self._receiver.get_audio_config(a_header)
+            # pylint: disable=protected-access
+            self._receiver._get_audio_codec(a_header)
             # self._schedule_congestion()
 
     def add_packet(self, msg: bytes):
         """Add Packet."""
-        packet = Packet.parse(msg)
+        packet = Packet.parse(msg, {"host_type": self._session.type})
         if len(self._queue) >= self._queue.maxlen:
             self._queue.clear()
             self._waiting = True
@@ -101,7 +103,7 @@ class AVHandler:
         _LOGGER.debug("AV Receiver Closed")
 
     def _handle(self, packet: AVPacket):
-        _LOGGER.debug(packet)
+        # _LOGGER.debug(packet)
         if not self._receiver:
             return
         if packet.type == AVPacket.Type.VIDEO:
@@ -114,7 +116,7 @@ class AVHandler:
         self._session.loop.call_later(0.5, self._send_congestion)
 
     def _send_congestion(self):
-        # TODO: Crashes remote play in game
+        # TODO: Use or don't use?
         now = time.time()
         if now - self._last_congestion > 0.2 and not self._session.is_stopped:
             self._session._sync_run_io(  # pylint: disable=protected-access
@@ -193,7 +195,7 @@ class AVStream:
         self._packets = []
         self._frame = packet.frame_index
         self._last_unit = -1
-        _LOGGER.debug("Started New Frame: %s", self.frame)
+        # _LOGGER.debug("Started New Frame: %s", self.frame)
 
     def _handle_missing_packet(self, index: int, unit_index: int):
         """Mark missing unit indexes as missing. Write null placeholder bytes for each."""
@@ -241,7 +243,7 @@ class AVStream:
         try:
             import pyjerasure  # pylint: disable=import-outside-toplevel
         except ModuleNotFoundError:
-            pass
+            return
         if not self._frame_bad_order and not self._missing:
             # Ignore FEC packets if all src packets received.
             return
