@@ -212,7 +212,15 @@ class Session:
         KEYBOARD_TEXT_CHANGE_RES = 0x24
         KEYBOARD_CLOSE_REQ = 0x25
 
-    class Protocol(asyncio.Protocol):
+    class ServerType(IntEnum):
+        """Server Type Enum."""
+
+        UNKNOWN = -1
+        PS4 = 0
+        PS4_PRO = 1
+        PS5 = 2
+
+    class _Protocol(asyncio.Protocol):
         """Protocol for session."""
 
         def __init__(self, session: Session):
@@ -224,6 +232,14 @@ class Session:
             _LOGGER.debug("Connected")
             self._transport = transport
 
+        def connection_lost(self, exc: Exception):
+            error = ""
+            if not self._session.is_stopped and not self._session.error:
+                error = str(exc) if exc else "Session disconnected unexpectedly"
+                self._session.error = error
+            _LOGGER.debug("Session Connection Lost; error: %s", error)
+            self._session.stop()
+
         def data_received(self, data: bytes):
             """Callback for data received."""
             # pylint: disable=protected-access
@@ -260,9 +276,9 @@ class Session:
         profile: Union[dict, UserProfile],
         loop: asyncio.AbstractEventLoop = None,
         receiver: AVReceiver = None,
-        resolution: Union[Resolution, str, int] = "360p",
+        resolution: Union[Resolution, str, int] = "720p",
         fps: Union[FPS, str, int] = "low",
-        quality: Union[Quality, str, int] = "very_low",
+        quality: Union[Quality, str, int] = "default",
         codec: str = "h264",
         hdr: bool = False,
     ):
@@ -272,7 +288,7 @@ class Session:
         self._profile = profile
         self._regist_data = {}
         self._session_id = b""
-        self._server_type = None
+        self._server_type = Session.ServerType.UNKNOWN
         self._type = ""
         self._regist_key = None
         self._rp_key = None
@@ -312,6 +328,12 @@ class Session:
             )
         self.set_receiver(receiver)
 
+    def _set_lowest_stream(self):
+        """Set stream parameters to lowest config possible."""
+        self._quality = Quality.VERY_LOW
+        self._resolution = Resolution.RESOLUTION_360P
+        self._fps = FPS.LOW
+
     def _init_profile(self, status: dict) -> bool:
         """Return True if Init profile."""
         mac_address = status.get("host-id")
@@ -403,9 +425,14 @@ class Session:
         server_type = response.headers.get("RP-Server-Type")
         if response.status_code != 200 or server_type is None:
             return False
-        self._server_type = int.from_bytes(
+        _server_type = int.from_bytes(
             self._cipher.decrypt(b64decode(server_type)), "little"
         )
+        try:
+            self._server_type = Session.ServerType(_server_type)
+        except ValueError:
+            _LOGGER.error("Unknown Server Type: %s", _server_type)
+
         _LOGGER.debug("Server Type: %s", self._server_type)
         self._sock = socket.fromfd(
             response.raw.fileno(), socket.AF_INET, socket.SOCK_STREAM
@@ -597,6 +624,10 @@ class Session:
         self._ready_event = asyncio.Event()
         self._stop_event = asyncio.Event()
 
+        if not self._receiver:
+            # Set Stream settings to lowest possible to reduce load
+            self._set_lowest_stream()
+
         self.events.on("av_ready", self._init_av_handler)
 
         if not self.loop:
@@ -621,7 +652,7 @@ class Session:
         _LOGGER.info("Session Auth Success")
         self._state = Session.State.RUNNING
         _, self._protocol = await self.loop.connect_accepted_socket(
-            lambda: Session.Protocol(self), self._sock
+            lambda: Session._Protocol(self), self._sock
         )
         await self._ready_event.wait()
         if autostart:
@@ -754,6 +785,12 @@ class Session:
     @property
     def resolution(self) -> Resolution:
         """Return resolution."""
+        if (
+            self.server_type == Session.ServerType.PS4
+            and self._resolution == Resolution.RESOLUTION_1080P
+        ):
+            # Downgrade resolution to 720p if PS4
+            return Resolution.RESOLUTION_720P
         return self._resolution
 
     @property
@@ -783,6 +820,11 @@ class Session:
             return StreamType.H264
         return self._stream_type
 
+    @property
+    def server_type(self) -> ServerType:
+        """Return Server Type."""
+        return self._server_type
+
     @property
     def receiver(self) -> AVReceiver:
         """Return AV Receiver."""
