@@ -12,6 +12,8 @@ from typing import TYPE_CHECKING
 from Cryptodome.Random import get_random_bytes
 from Cryptodome.Util.strxor import strxor
 
+from pyremoteplay.const import TYPE_PS5
+
 from .av import AVHandler
 from .crypt import StreamECDH
 from .protobuf import ProtoHandler
@@ -67,6 +69,9 @@ class RPStream:
             """Callback for data received"""
             self.stream.handle(data)
 
+        def error_received(self, exc):
+            _LOGGER.error("Stream Protocol Error: %s", exc)
+
         def sendto(self, data, addr):
             """Send data."""
             self.transport.sendto(data, addr)
@@ -209,11 +214,18 @@ class RPStream:
             tag=self._tag_local,
             tsn=ack_tsn,
         )
+        _LOGGER.debug("Sent Data ACK: %s", ack_tsn)
         self.send(msg.bytes(self._cipher, False, DATA_ACK_LENGTH))
 
     def send_feedback(self, feedback_type: int, sequence: int, data=b"", state=None):
         """Send feedback packet."""
-        msg = FeedbackPacket(feedback_type, sequence=sequence, data=data, state=state)
+        msg = FeedbackPacket(
+            feedback_type,
+            sequence=sequence,
+            data=data,
+            state=state,
+            host_type=self._session.type,
+        )
         self.send(msg.bytes(self._cipher, True))
 
     def send_congestion(self, received: int, lost: int):
@@ -266,7 +278,9 @@ class RPStream:
             key_pos = packet.header.key_pos
             packet.header.gmac = packet.header.key_pos = 0
             if self._verify_gmac:
-                gmac = self._cipher.verify_gmac(packet.bytes(), key_pos, _gmac)
+                verified = self._cipher.verify_gmac(packet.bytes(), key_pos, _gmac)
+                if not verified:
+                    _LOGGER.error("GMAC Mismatch")
 
         if packet.chunk.type == Chunk.Type.INIT_ACK:
             self._recv_init(packet)
@@ -276,6 +290,8 @@ class RPStream:
             self._recv_data_ack(packet)
         elif packet.chunk.type == Chunk.Type.DATA:
             self._recv_data(packet)
+        else:
+            _LOGGER.debug("RECV Unhandled chunk type: %s", packet.chunk.type)
 
     def _recv_init(self, packet: Packet):
         """Handle Init."""
@@ -312,12 +328,14 @@ class RPStream:
     def _send_big(self):
         chunk_flag = channel = 1
         if not self._is_test:
+            version = 12 if self._session.type == TYPE_PS5 else 9
+            _LOGGER.debug("Using Version: %s", version)
             self._ecdh = StreamECDH()
             launch_spec = self._format_launch_spec(self._ecdh.handshake_key)
             encrypted_key = bytes(4)
             ecdh_pub_key = self._ecdh.public_key
             ecdh_sig = self._ecdh.public_sig
-            client_version = 9
+            client_version = version
         else:
             launch_spec = b""
             encrypted_key = b""
@@ -339,6 +357,7 @@ class RPStream:
     def _format_launch_spec(self, handshake_key: bytes, format_type=None) -> bytes:
         launch_spec = get_launch_spec(
             handshake_key=handshake_key,
+            host_type=self._session.type,
             resolution=self._session.resolution,
             fps=self._session.fps,
             quality=self._session.quality,
