@@ -1,13 +1,16 @@
 """Stream Packets for pyremoteplay."""
+from __future__ import annotations
 import abc
 import json
 import logging
 from base64 import b64encode
+import dataclasses
 from enum import IntEnum
 from struct import pack, pack_into, unpack_from
 from typing import Iterable, Union
+from math import sqrt
 
-from .const import Quality, Resolution, FPS, StreamType
+from .const import TYPE_PS4, TYPE_PS5, Quality, Resolution, FPS, StreamType
 from .crypt import StreamCipher
 
 _LOGGER = logging.getLogger(__name__)
@@ -18,6 +21,9 @@ OUTBOUND_STREAMS = 0x64
 INBOUND_STREAMS = 0x64
 
 
+_I_SQRT = 1 / sqrt(2)
+
+
 class UnexpectedMessage(Exception):
     """Message not Expected."""
 
@@ -58,7 +64,7 @@ def __get_launch_spec() -> dict:
             "language": "sp",
             "acceptButton": "X",
             "connectedControllers": ["xinput", "ds3", "ds4"],
-            "yuvCoefficient": "bt709",  # Changed from bt601
+            "yuvCoefficient": "bt601",
             "videoEncoderProfile": "hw4.1",
             "audioEncoderProfile": "audio1",
         },
@@ -68,7 +74,6 @@ def __get_launch_spec() -> dict:
             "region": "US",
             "languagesUsed": ["en", "jp"],
         },
-        "adaptiveStreamMode": "resize",
         "videoCodec": "",
         "dynamicRange": "",
         "handshakeKey": None,
@@ -77,6 +82,7 @@ def __get_launch_spec() -> dict:
 
 def get_launch_spec(
     handshake_key: bytes,
+    host_type: str,
     resolution: Resolution,
     fps: FPS,
     quality: Quality,
@@ -105,6 +111,10 @@ def get_launch_spec(
     launch_spec["videoCodec"] = "hevc" if codec == "hevc" else "avc"
     launch_spec["dynamicRange"] = "HDR" if hdr else "SDR"
     launch_spec["handshakeKey"] = b64encode(handshake_key).decode()
+
+    if host_type == TYPE_PS5:
+        launch_spec["requestGameSpecification"]["adaptiveStreamMode"] = "resize"
+
     launch_spec = json.dumps(launch_spec)
     launch_spec = launch_spec.replace(" ", "")  # minify
     launch_spec = launch_spec.replace(":0.001,", ":0.001000,")  # Add three zeros
@@ -161,6 +171,75 @@ class StickState:
         self._y = self.__scale_normalize(value)
 
 
+class MotionState:
+    """Controller Motion State."""
+
+    @dataclasses.dataclass(repr=True)
+    class GyroState:
+        """Gyro State."""
+
+        x: float = 0.0
+        y: float = 0.0
+        z: float = 0.0
+
+        def values(self) -> tuple[float]:
+            """Return values."""
+            return dataclasses.astuple(self)
+
+        def max(self) -> float:
+            """Return max value."""
+            return 30.0
+
+    @dataclasses.dataclass(repr=True)
+    class AccelState:
+        """Accel State"""
+
+        x: float = 0.0
+        y: float = 1.0
+        z: float = 0.0
+
+        def values(self) -> tuple[float]:
+            """Return values."""
+            return dataclasses.astuple(self)
+
+        def max(self) -> float:
+            """Return max value."""
+            return 5.0
+
+    @dataclasses.dataclass(repr=True)
+    class OrientState:
+        """Orient State."""
+
+        x: float = 0.0
+        y: float = 0.0
+        z: float = 0.0
+        w: float = 1.0
+
+        def values(self) -> tuple[float]:
+            """Return values."""
+            return dataclasses.astuple(self)
+
+    def __init__(self):
+        self._gyro = self.GyroState()
+        self._accel = self.AccelState()
+        self._orient = self.OrientState()
+
+    @property
+    def gyro(self) -> GyroState:
+        """Return Gyro State."""
+        return self._gyro
+
+    @property
+    def accel(self) -> AccelState:
+        """Return Accel State."""
+        return self._accel
+
+    @property
+    def orient(self) -> OrientState:
+        """Return Orient State."""
+        return self._orient
+
+
 class ControllerState:
     """State of both controller sticks."""
 
@@ -183,6 +262,7 @@ class ControllerState:
     ):
         self._left: StickState = self.__state_setter(left)
         self._right: StickState = self.__state_setter(right)
+        self._motion: MotionState = MotionState()
 
     def __state_setter(
         self, state: Union[StickState, Iterable[Union[int, float]], None]
@@ -220,6 +300,11 @@ class ControllerState:
         """Set right State."""
         self._right = self.__state_setter(state)
 
+    @property
+    def motion(self) -> MotionState:
+        """Return motion state."""
+        return self._motion
+
 
 class PacketSection(abc.ABC):
     """Abstract Packet Section."""
@@ -508,13 +593,14 @@ class Packet(AbstractPacket):
         return 0
 
     @staticmethod
-    def parse(buf: bytes, params=None):
+    def parse(buf: bytes, params: Union[dict, None] = None):
         """Return new instance from bytes.
 
         Params is updated when parsing each section.
         """
         buf = bytearray(buf)
-        params = {}
+        if params is None:
+            params = {}
         av_mask = Packet.is_av(buf[:1])
         if av_mask:
             return AVPacket(av_mask, buf, **params)
@@ -592,6 +678,7 @@ class AVPacket(AbstractPacket):
     # pylint: disable=unused-argument
     def __init__(self, av_type: int, buf: bytearray, **kwargs):
         super().__init__(av_type)
+        self._host_type = kwargs.get("host_type") or TYPE_PS4
         self._has_nalu = (unpack_from("!B", buf, 0)[0] >> 4) & 1 != 0
         self._index = unpack_from("!H", buf, 1)[0]
         self._frame_index = unpack_from("!H", buf, 3)[0]
@@ -616,6 +703,9 @@ class AVPacket(AbstractPacket):
             # Unknown ushort at 18
             self._nalu = buf[18 + offset + 1 : 18 + offset + 3]
             offset += 3
+
+        if self.type == Header.Type.AUDIO and self._host_type == TYPE_PS5:
+            offset += 1
         self._data = buf[18 + offset :]
 
         self._get_frame_meta()
@@ -781,29 +871,16 @@ class FeedbackHeader(PacketSection):
 class FeedbackState(PacketSection):
     """Feedback Event."""
 
-    LENGTH = 25
+    LENGTH = 28
+    _MOTION_LENGTH = 17
 
-    PREFIX = bytes(
-        [
-            0xA0,
-            0xFF,
-            0x7F,
-            0xFF,
-            0x7F,
-            0xFF,
-            0x7F,
-            0xFF,
-            0x7F,
-            0x99,
-            0x99,
-            0xFF,
-            0x7F,
-            0xFE,
-            0xF7,
-            0xEF,
-            0x1F,
-        ]
-    )
+    # fmt: off
+    _MOTION_IDLE = bytes([
+        0xA0, 0xFF, 0x7F, 0xFF, 0x7F, 0xFF, 0x7F, 0xFF,
+        0x7F, 0x99, 0x99, 0xFF, 0x7F, 0xFE, 0xF7, 0xEF,
+        0x1F,
+    ])
+    # fmt: on
 
     class Type(IntEnum):
         """Enums for State."""
@@ -816,19 +893,67 @@ class FeedbackState(PacketSection):
     def __init__(self, state_type, **kwargs):
         super().__init__(state_type)
         self.state: ControllerState = kwargs.get("state") or ControllerState()
+        self.host_type = kwargs.get("host_type") or TYPE_PS4
+
+    def _get_quaternion(self) -> int:
+        values = self.state.motion.orient.values()
+        abs_values = [abs(val) for val in values]
+        largest_index = abs_values.index(max(abs_values))
+        largest = values[largest_index]
+        result = 1 if largest < 0.0 else 0
+        result |= largest_index << 1
+        for index, value in enumerate(values):
+            if index == largest_index:
+                continue
+            if value < -_I_SQRT:
+                value = -_I_SQRT
+            elif value > _I_SQRT:
+                value = _I_SQRT
+            value += _I_SQRT
+            value *= 0x01FF / (2.0 * _I_SQRT)
+            result |= int(value) << (3 + index * 9)
+        return result
+
+    def _pack_motion_state(self) -> bytes:
+        buf = bytearray(self._MOTION_LENGTH)
+        offset = 0
+        pack_into("<B", buf, offset, 0xA0)
+        offset += 1
+        states = [self.state.motion.gyro, self.state.motion.accel]
+        for state in states:
+            max_val = state.max()
+            for value in state.values():
+                val = int(0xFFFF * (value + max_val) / (max_val + max_val))
+                pack_into("<H", buf, offset, val)
+                offset += 2
+        pack_into("<I", buf, offset, self._get_quaternion())
+        return bytes(buf)
 
     def pack(self, buf: bytearray):
         """Pack compiled bytes."""
         pack_into(
-            "!17shhhh",
+            f"!{self._MOTION_LENGTH}shhhh",
             buf,
             FeedbackHeader.LENGTH,
-            self.PREFIX,
+            # self._pack_motion_state(), # TODO: implement
+            self._MOTION_IDLE,
             self.state.left.x,
             self.state.left.y,
             self.state.right.x,
             self.state.right.y,
         )
+        if self.host_type == TYPE_PS5:
+            pack_into(
+                "!hB",
+                buf,
+                FeedbackState.LENGTH - 3,
+                0,
+                1,  # 1 DS4; 0 DualSense
+            )
+
+    @property
+    def length(self) -> int:
+        return self.LENGTH if self.host_type == TYPE_PS5 else self.LENGTH - 3
 
 
 class FeedbackEvent(PacketSection):
@@ -923,7 +1048,7 @@ class FeedbackPacket(AbstractPacket):
         data_length = (
             len(self.data)
             if self.header.type == FeedbackHeader.Type.EVENT
-            else FeedbackState.LENGTH
+            else self.chunk.length  # Feedback STATE
         )
         length = data_length + FeedbackHeader.LENGTH
         buf = bytearray(length)
